#!/bin/bash

# define main function
MAIN_INSTALL="main_install"
MAIN_UPGRADE="main_upgrade"
MAIN_UNINST="main_uninst"
# define each page's name
PAGE_MARIADB5="page_mariadb5"
PAGE_MARIADB10="page_mariadb10"
PAGE_RESTORE="page_restore"
PAGE_DB="page_db"
PAGE_UNINST="page_uninst"
DEFAULT="enu"
# set out file name
OUTPUT_INSTALL="install_uifile"
OUTPUT_UPGRADE="upgrade_uifile"
OUTPUT_UNINST="uninstall_uifile"
INIT_DB_NAME="" # need to be overwrited
INIT_DB_USER="" # need to be overwrited
INFO_FILE="" # need to be overwrited
BACKUP_PATH="" # need to be overwrited
# here to define what page is necessary for each stage: install, upgrade, and uninstall
PAGE_INST="$PAGE_MARIADB5 $PAGE_RESTORE $PAGE_MARIADB10 $PAGE_DB $MAIN_INSTALL"
PAGE_UPGRADE="$PAGE_MARIADB5 $PAGE_MARIADB10 $PAGE_DB $MAIN_UPGRADE"
PAGE_UNINST="$PAGE_UNINST $MAIN_UNINST"
MIGRATE_DB_VERSION="-1"

MainCreate()
{
	local output="$1"
	local pages="$2"
	local main_function="$3"
	local lan
	for dir in lang/*; do
		lan=$(basename "$dir")
		cat "wizard_common" "wizard_customized" > "${output}_${lan}.sh"
		CustomWizCreate "${output}_${lan}.sh"
		cat "lang/$lan" $pages "$main_function" >> "${output}_${lan}.sh"
	done
	chmod +x "${output}"*.sh
	cp -a "${output}_${DEFAULT}.sh" "${output}.sh"
}

NeedMigrateDB()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -le "$MIGRATE_DB_VERSION" ]; then
		return 0
	fi
	return 1
}

CustomHasRunWebsiteSetup()
{
	local path="$1"
	[ -s "$path/$CONF_FILE" ]
}

CustomWizCreate()
{
	:
}

CustomSetBackupPath()
{
	BACKUP_PATH="$(CustomGetBackupPkgPath)"
}

CustomMainInstall()
{
	:
}

CustomGetValueFromMetaFile()
{
	local key="$1"
	local backup_info="/var/packages/.$PKG_DIR.conf"
	local syno_conf="/var/packages/$PKG_NAME/synology.conf"
	local record_file=""

	if [ -s "$INFO_FILE" ]; then
		record_file="$INFO_FILE"
	elif [ -s "$backup_info" ]; then
		record_file="$backup_info"
	elif [ -s "$syno_conf" ]; then
		record_file="$syno_conf"
	fi
	get_key_value "$record_file" "$key"
}

CustomGetBackupPrefix()
{
	local backup_prefix="/$(readlink /var/services/web | cut -d/ -f2)/@appstore/.$PKG_DIR"
	echo "$backup_prefix"
}

CustomGetBackupPkgPath()
{
	local backup_pkg_path="$(CustomGetBackupPrefix)/$PKG_DIR"
	echo "$backup_pkg_path"
}

# we define pages in fuction not in global variables,
# because functions in wizard steps will be merged after calling MainCreate.
CustomSetPages()
{
	NEW_INSTALL_PAGES="$(PageM10),$(PageDB)"
	RESTORE_PAGES="$(PageRestore),$(PageM10),$(PageDB)"
	MIGRATE_PAGES="$(PageM5),$(PageRestore),$(PageM10),$(PageDB)"
}

CustomParseDBConf()
{
	:
}

CustomGetDBName()
{
	:
}

CustomGetDBUser()
{
	:
}

CustomGetDBPass()
{
	:
}

CustomPageActivate()
{
	:
}
#!/bin/bash

PKG_NAME="MediaWiki"
PKG_DIR="mediawiki"
WEBSITE_ROOT="/var/services/web/$PKG_DIR"
INFO_FILE="/usr/syno/etc/packages/$PKG_NAME/$PKG_DIR.conf"
CONF_FILE="LocalSettings.php"
MIGRATE_DB_VERSION="0122"
INIT_DB_NAME="mediawiki"
INIT_DB_USER="mediawiki_user"

CustomGetBackupPkgPath()
{
	local backup_pkg_path=$(CustomGetBackupPrefix)
	# Before pkg-version 0020, we backup mediawiki into mediawiki-1.19.0
	if [ ! -d "$backup_pkg_path" ] && [ -d "$backup_pkg_path/source" ]; then
		backup_pkg_path=$backup_pkg_path/source
	elif [ ! -d "$backup_pkg_path" ] && [ -d "$backup_pkg_path/mediawiki-1.19.0" ]; then
		backup_pkg_path=$backup_pkg_path/mediawiki-1.19.0
	fi
	echo "$backup_pkg_path"
}

CustomParseDBConf()
{
	local path="$1"
	local db_info="$2"
	local grep_info="$3"
	local info_output=$(CustomGetValueFromMetaFile "$db_info")

	if [ -z "$info_output" ] && [ -s "$path/$CONF_FILE" ]; then
		info_output=$(grep "$grep_info" "$path/$CONF_FILE" | cut -d'"' -f2)
	fi

	echo "$info_output"
}

CustomGetDBName()
{
	CustomParseDBConf "$1" "db_name" "^\$wgDBname "
}

CustomGetDBUser()
{
	CustomParseDBConf "$1" "db_user" "^\$wgDBuser "
}

CustomGetDBPass()
{
	CustomParseDBConf "$1" "db_pass" "^\$wgDBpassword "
}
wizard_db_settings="Konfigurera databasen MediaWiki"
wizard_db_name_desc="Ange databasnamnet för MediaWiki databas. "
wizard_db_name_label="Databasnamn"
wizard_m10_desc="Vänligen ange administratörens autentiseringsuppgifter för MariaDB 10 för att fortsätta med databasinställningar för MediaWiki."
wizard_admin_acc="Konto"
wizard_admin_pass="Lösenord"
wizard_set_db_desc="Vänligen skapa ett nytt exklusivt databaskonto för MediaWiki för att använda befintliga/nya data."
wizard_set_data_title="Konfigurera MediaWiki"
wizard_found_backup="Vänligen välj en metod för att importera data."
wizard_decide_restore="Databasen MediaWiki finns redan.<br>Vänligen markera någon av följande åtgärder:"
wizard_restore="Använd befintliga data"
wizard_create_new="Ren installation (alla befintliga data, inklusive konfigurationsfiler och databasen tas bort)."
wizard_db_user_account_desc="Användarkonto för databas"
wizard_db_user_password_desc="Lösenord för databasanvändare"
wizard_migrate_note="Den senaste versionen av MediaWiki har bara stöd för MariaDB 10.<br>Systemet har upptäckt en befintlig MariaDB 5-databas.<br>I nästa steg blir din databas migrerad från MariaDB 5 till MariaDB 10.<br><br>Vänligen ange autentiseringsuppgifter för administratören för MariaDB 5 för att fortsätta med installationen."
wizard_migrate_title="Migrera databasen MediaWiki"
wizard_remove_msql_title="Ta bort MediaWiki-databas"
wizard_remove_msql_desc="Om databasen MediaWiki tas bort raderas alla data."
wizard_msql_password_desc_remove="Vänligen ange autentiseringsuppgifter för databasadministratören för att ta bort databasen för MediaWiki."
wizard_admin_acc="Konto"
wizard_admin_pass="Lösenord"
PageUninst()
{
cat << EOF
{
	"step_title": "$wizard_remove_msql_title",
	"items": [{
		"type": "multiselect",
		"desc": "$wizard_remove_msql_desc",
		"subitems": [{
			"key": "pkgwizard_remove_mysql",
			"desc": "$wizard_remove_msql_title"
		}]
	}, {
		"type": "textfield",
		"desc": "$wizard_msql_password_desc_remove",
		"subitems": [{
			"key": "wizard_m10_acc",
			"desc": "$wizard_admin_acc",
			"defaultValue": "root"
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "wizard_m10_pass",
			"desc": "$wizard_admin_pass"
		}]
	}]
}
EOF
}
main()
{
	local uninst_page="$(PageUninst)"
	echo "[$uninst_page]" > "${SYNOPKG_TEMP_LOGFILE}"

	return 0
}

main "$@"
main()
{
	local uninst_page="$(PageUninst)"
	echo "[$uninst_page]" > "${SYNOPKG_TEMP_LOGFILE}"

	return 0
}

main "$@"
