#!/bin/bash

# define main function
MAIN_INSTALL="main_install"
MAIN_UPGRADE="main_upgrade"
MAIN_UNINST="main_uninst"
# define each page's name
PAGE_MARIADB5="page_mariadb5"
PAGE_MARIADB10="page_mariadb10"
PAGE_RESTORE="page_restore"
PAGE_DB="page_db"
PAGE_UNINST="page_uninst"
DEFAULT="enu"
# set out file name
OUTPUT_INSTALL="install_uifile"
OUTPUT_UPGRADE="upgrade_uifile"
OUTPUT_UNINST="uninstall_uifile"
INIT_DB_NAME="" # need to be overwrited
INIT_DB_USER="" # need to be overwrited
INFO_FILE="" # need to be overwrited
BACKUP_PATH="" # need to be overwrited
# here to define what page is necessary for each stage: install, upgrade, and uninstall
PAGE_INST="$PAGE_MARIADB5 $PAGE_RESTORE $PAGE_MARIADB10 $PAGE_DB $MAIN_INSTALL"
PAGE_UPGRADE="$PAGE_MARIADB5 $PAGE_MARIADB10 $PAGE_DB $MAIN_UPGRADE"
PAGE_UNINST="$PAGE_UNINST $MAIN_UNINST"
MIGRATE_DB_VERSION="-1"

MainCreate()
{
	local output="$1"
	local pages="$2"
	local main_function="$3"
	local lan
	for dir in lang/*; do
		lan=$(basename "$dir")
		cat "wizard_common" "wizard_customized" > "${output}_${lan}.sh"
		CustomWizCreate "${output}_${lan}.sh"
		cat "lang/$lan" $pages "$main_function" >> "${output}_${lan}.sh"
	done
	chmod +x "${output}"*.sh
	cp -a "${output}_${DEFAULT}.sh" "${output}.sh"
}

NeedMigrateDB()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -le "$MIGRATE_DB_VERSION" ]; then
		return 0
	fi
	return 1
}

CustomHasRunWebsiteSetup()
{
	local path="$1"
	[ -s "$path/$CONF_FILE" ]
}

CustomWizCreate()
{
	:
}

CustomSetBackupPath()
{
	BACKUP_PATH="$(CustomGetBackupPkgPath)"
}

CustomMainInstall()
{
	:
}

CustomGetValueFromMetaFile()
{
	local key="$1"
	local backup_info="/var/packages/.$PKG_DIR.conf"
	local syno_conf="/var/packages/$PKG_NAME/synology.conf"
	local record_file=""

	if [ -s "$INFO_FILE" ]; then
		record_file="$INFO_FILE"
	elif [ -s "$backup_info" ]; then
		record_file="$backup_info"
	elif [ -s "$syno_conf" ]; then
		record_file="$syno_conf"
	fi
	get_key_value "$record_file" "$key"
}

CustomGetBackupPrefix()
{
	local backup_prefix="/$(readlink /var/services/web | cut -d/ -f2)/@appstore/.$PKG_DIR"
	echo "$backup_prefix"
}

CustomGetBackupPkgPath()
{
	local backup_pkg_path="$(CustomGetBackupPrefix)/$PKG_DIR"
	echo "$backup_pkg_path"
}

# we define pages in fuction not in global variables,
# because functions in wizard steps will be merged after calling MainCreate.
CustomSetPages()
{
	NEW_INSTALL_PAGES="$(PageM10),$(PageDB)"
	RESTORE_PAGES="$(PageRestore),$(PageM10),$(PageDB)"
	MIGRATE_PAGES="$(PageM5),$(PageRestore),$(PageM10),$(PageDB)"
}

CustomParseDBConf()
{
	:
}

CustomGetDBName()
{
	:
}

CustomGetDBUser()
{
	:
}

CustomGetDBPass()
{
	:
}

CustomPageActivate()
{
	:
}
#!/bin/bash

PKG_NAME="MediaWiki"
PKG_DIR="mediawiki"
WEBSITE_ROOT="/var/services/web/$PKG_DIR"
INFO_FILE="/usr/syno/etc/packages/$PKG_NAME/$PKG_DIR.conf"
CONF_FILE="LocalSettings.php"
MIGRATE_DB_VERSION="0122"
INIT_DB_NAME="mediawiki"
INIT_DB_USER="mediawiki_user"

CustomGetBackupPkgPath()
{
	local backup_pkg_path=$(CustomGetBackupPrefix)
	# Before pkg-version 0020, we backup mediawiki into mediawiki-1.19.0
	if [ ! -d "$backup_pkg_path" ] && [ -d "$backup_pkg_path/source" ]; then
		backup_pkg_path=$backup_pkg_path/source
	elif [ ! -d "$backup_pkg_path" ] && [ -d "$backup_pkg_path/mediawiki-1.19.0" ]; then
		backup_pkg_path=$backup_pkg_path/mediawiki-1.19.0
	fi
	echo "$backup_pkg_path"
}

CustomParseDBConf()
{
	local path="$1"
	local db_info="$2"
	local grep_info="$3"
	local info_output=$(CustomGetValueFromMetaFile "$db_info")

	if [ -z "$info_output" ] && [ -s "$path/$CONF_FILE" ]; then
		info_output=$(grep "$grep_info" "$path/$CONF_FILE" | cut -d'"' -f2)
	fi

	echo "$info_output"
}

CustomGetDBName()
{
	CustomParseDBConf "$1" "db_name" "^\$wgDBname "
}

CustomGetDBUser()
{
	CustomParseDBConf "$1" "db_user" "^\$wgDBuser "
}

CustomGetDBPass()
{
	CustomParseDBConf "$1" "db_pass" "^\$wgDBpassword "
}
wizard_db_settings="MediaWiki Veritabanını Kur"
wizard_db_name_desc="Lütfen MediaWiki veritabanı için veritabanı adını girin. "
wizard_db_name_label="Veritabanı adı"
wizard_m10_desc="Lütfen MariaDB 10'un yönetici bilgilerini girerek MediaWiki veritabanı ayarlarına geçin."
wizard_admin_acc="Hesap"
wizard_admin_pass="Parola"
wizard_set_db_desc="Mevcut/yeni verileri kullanmak üzere MediaWiki için yeni bir özel veritabanı hesabı oluşturun."
wizard_set_data_title="MediaWiki kur"
wizard_found_backup="Lütfen verileri içeri aktarmak için bir yöntem seçin."
wizard_decide_restore="MediaWiki veritabanı zaten mevcut.<br>Lütfen aşağıdaki eylemlerden birisini seçin:"
wizard_restore="Mevcut verileri kullan"
wizard_create_new="Kurulumu kaldır (Tüm mevcut veriler, yapılandırma dosyaları ve veritabanı dahil, kaldırılacaktır.)"
wizard_db_user_account_desc="Veritabanı kullanıcı hesabı"
wizard_db_user_password_desc="Veritabanı kullanıcı parolası"
wizard_migrate_note="MediaWiki son sürümü sadece MariaDB 10 destekler.<br>Sistem mevcut bir MariaDB 5 veritabanını algıladı.<br>Sonraki adımda, veritabanınız MariaDB 5'den MariaDB 10'a taşınacaktır.<br><br>Lütfen kuruluma geçmek için MariaDB 5 yönetici bilgilerini girin."
wizard_migrate_title="MediaWiki Veritabanını Taşı"
wizard_remove_msql_title="MediaWiki veritabanını kaldır"
wizard_remove_msql_desc="MediaWiki veritabanı kaldırılırsa, tüm veriler silinecektir."
wizard_msql_password_desc_remove="MediaWiki veritabanını kaldırmak için lütfen veritabanı yöneticisinin bilgilerini girin."
wizard_admin_acc="Hesap"
wizard_admin_pass="Parola"
PageUninst()
{
cat << EOF
{
	"step_title": "$wizard_remove_msql_title",
	"items": [{
		"type": "multiselect",
		"desc": "$wizard_remove_msql_desc",
		"subitems": [{
			"key": "pkgwizard_remove_mysql",
			"desc": "$wizard_remove_msql_title"
		}]
	}, {
		"type": "textfield",
		"desc": "$wizard_msql_password_desc_remove",
		"subitems": [{
			"key": "wizard_m10_acc",
			"desc": "$wizard_admin_acc",
			"defaultValue": "root"
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "wizard_m10_pass",
			"desc": "$wizard_admin_pass"
		}]
	}]
}
EOF
}
main()
{
	local uninst_page="$(PageUninst)"
	echo "[$uninst_page]" > "${SYNOPKG_TEMP_LOGFILE}"

	return 0
}

main "$@"
main()
{
	local uninst_page="$(PageUninst)"
	echo "[$uninst_page]" > "${SYNOPKG_TEMP_LOGFILE}"

	return 0
}

main "$@"
